{% extends "admin/change_list.html" %}

{% block object-tools %}
    <div>
        <form action="../rule/test_rules/" method="POST">
            {% csrf_token %}
            <div>
                <label>
                    Test ip
                    <input name="test_ip" />
                    (leave empty to use current ip address)
                </label>
            </div>
            <div>
                <label>
                    Test path
                    <input type="path" name="test_path" />
                    (leave empty to test ip addresses only)
                </label>
            </div>
            <button type="submit">Test rules</button>
        </form>
    </div>
    <br />
    <div>
        <a href="../rule/simulate_rules/">simulate rules</a>
    </div>
    <br />
    {{ block.super }}
{% endblock %}
